package sun.awt;

import java.util.Properties;
import java.util.Iterator;
import java.util.Map;
import java.util.Arrays;
import sun.font.CompositeFontDescriptor;
import java.nio.charset.CharsetEncoder;
import java.util.Vector;
import java.io.DataOutputStream;
import java.io.OutputStream;
import java.io.DataInputStream;
import java.io.InputStream;
import java.io.IOException;
import java.io.FileInputStream;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.nio.charset.Charset;
import sun.font.FontUtilities;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Set;
import java.io.File;
import sun.font.SunFontManager;
import sun.util.logging.PlatformLogger;
import java.util.Hashtable;
import java.util.Locale;

public abstract class FontConfiguration
{
    protected static String osVersion;
    protected static String osName;
    protected static String encoding;
    protected static Locale startupLocale;
    protected static Hashtable localeMap;
    private static FontConfiguration fontConfig;
    private static PlatformLogger logger;
    protected static boolean isProperties;
    protected SunFontManager fontManager;
    protected boolean preferLocaleFonts;
    protected boolean preferPropFonts;
    private File fontConfigFile;
    private boolean foundOsSpecificFile;
    private boolean inited;
    private String javaLib;
    private static short stringIDNum;
    private static short[] stringIDs;
    private static StringBuilder stringTable;
    public static boolean verbose;
    private short initELC;
    private Locale initLocale;
    private String initEncoding;
    private String alphabeticSuffix;
    private short[][][] compFontNameIDs;
    private int[][][] compExclusions;
    private int[] compCoreNum;
    private Set<Short> coreFontNameIDs;
    private Set<Short> fallbackFontNameIDs;
    protected static final int NUM_FONTS = 5;
    protected static final int NUM_STYLES = 4;
    protected static final String[] fontNames;
    protected static final String[] publicFontNames;
    protected static final String[] styleNames;
    protected static String[] installedFallbackFontFiles;
    protected HashMap reorderMap;
    private Hashtable charsetRegistry;
    private FontDescriptor[][][] fontDescriptors;
    HashMap<String, Boolean> existsMap;
    private int numCoreFonts;
    private String[] componentFonts;
    HashMap<String, String> filenamesMap;
    HashSet<String> coreFontFileNames;
    private static final int HEAD_LENGTH = 20;
    private static final int INDEX_scriptIDs = 0;
    private static final int INDEX_scriptFonts = 1;
    private static final int INDEX_elcIDs = 2;
    private static final int INDEX_sequences = 3;
    private static final int INDEX_fontfileNameIDs = 4;
    private static final int INDEX_componentFontNameIDs = 5;
    private static final int INDEX_filenames = 6;
    private static final int INDEX_awtfontpaths = 7;
    private static final int INDEX_exclusions = 8;
    private static final int INDEX_proportionals = 9;
    private static final int INDEX_scriptFontsMotif = 10;
    private static final int INDEX_alphabeticSuffix = 11;
    private static final int INDEX_stringIDs = 12;
    private static final int INDEX_stringTable = 13;
    private static final int INDEX_TABLEEND = 14;
    private static final int INDEX_fallbackScripts = 15;
    private static final int INDEX_appendedfontpath = 16;
    private static final int INDEX_version = 17;
    private static short[] head;
    private static short[] table_scriptIDs;
    private static short[] table_scriptFonts;
    private static short[] table_elcIDs;
    private static short[] table_sequences;
    private static short[] table_fontfileNameIDs;
    private static short[] table_componentFontNameIDs;
    private static short[] table_filenames;
    protected static short[] table_awtfontpaths;
    private static short[] table_exclusions;
    private static short[] table_proportionals;
    private static short[] table_scriptFontsMotif;
    private static short[] table_alphabeticSuffix;
    private static short[] table_stringIDs;
    private static char[] table_stringTable;
    private HashMap<String, Short> reorderScripts;
    private static String[] stringCache;
    private static final int[] EMPTY_INT_ARRAY;
    private static final String[] EMPTY_STRING_ARRAY;
    private static final short[] EMPTY_SHORT_ARRAY;
    private static final String UNDEFINED_COMPONENT_FONT = "unknown";
    
    public FontConfiguration(SunFontManager fontManager) {
        this.initELC = -1;
        this.compFontNameIDs = new short[5][4][];
        this.compExclusions = new int[5][][];
        this.compCoreNum = new int[5];
        this.coreFontNameIDs = new HashSet();
        this.fallbackFontNameIDs = new HashSet();
        this.reorderMap = null;
        this.charsetRegistry = new Hashtable(5);
        this.fontDescriptors = new FontDescriptor[5][4][];
        this.numCoreFonts = -1;
        this.componentFonts = null;
        this.filenamesMap = new HashMap();
        this.coreFontFileNames = new HashSet();
        if (FontUtilities.debugFonts()) {
            FontUtilities.getLogger().info("Creating standard Font Configuration");
        }
        if (FontUtilities.debugFonts() && FontConfiguration.logger == null) {
            FontConfiguration.logger = PlatformLogger.getLogger("sun.awt.FontConfiguration");
        }
        this.fontManager = fontManager;
        this.setOsNameAndVersion();
        this.setEncoding();
        this.findFontConfigFile();
    }
    
    public synchronized boolean init() {
        if (!this.inited) {
            this.preferLocaleFonts = false;
            this.preferPropFonts = false;
            this.setFontConfiguration();
            this.readFontConfigFile(this.fontConfigFile);
            this.initFontConfig();
            this.inited = true;
        }
        return true;
    }
    
    public FontConfiguration(SunFontManager fontManager, boolean preferLocaleFonts, boolean preferPropFonts) {
        this.initELC = -1;
        this.compFontNameIDs = new short[5][4][];
        this.compExclusions = new int[5][][];
        this.compCoreNum = new int[5];
        this.coreFontNameIDs = new HashSet();
        this.fallbackFontNameIDs = new HashSet();
        this.reorderMap = null;
        this.charsetRegistry = new Hashtable(5);
        this.fontDescriptors = new FontDescriptor[5][4][];
        this.numCoreFonts = -1;
        this.componentFonts = null;
        this.filenamesMap = new HashMap();
        this.coreFontFileNames = new HashSet();
        this.fontManager = fontManager;
        if (FontUtilities.debugFonts()) {
            FontUtilities.getLogger().info("Creating alternate Font Configuration");
        }
        this.preferLocaleFonts = preferLocaleFonts;
        this.preferPropFonts = preferPropFonts;
        this.initFontConfig();
    }
    
    protected void setOsNameAndVersion() {
        FontConfiguration.osName = System.getProperty("os.name");
        FontConfiguration.osVersion = System.getProperty("os.version");
    }
    
    private void setEncoding() {
        FontConfiguration.encoding = Charset.defaultCharset().name();
        FontConfiguration.startupLocale = SunToolkit.getStartupLocale();
    }
    
    public boolean foundOsSpecificFile() {
        return this.foundOsSpecificFile;
    }
    
    public boolean fontFilesArePresent() {
        this.init();
        return (boolean)AccessController.doPrivileged(new PrivilegedAction() {
            
            @Override
            public Object run() {
                try {
                    return Boolean.valueOf(new File(fileName).exists());
                }
                catch (Exception ex) {
                    return Boolean.valueOf(false);
                }
            }
        });
    }
    
    private void findFontConfigFile() {
        this.foundOsSpecificFile = true;
        String property = System.getProperty("java.home");
        if (property == null) {
            throw new Error("java.home property not set");
        }
        this.javaLib = property + File.separator + "lib";
        String property2 = System.getProperty("sun.awt.fontconfig");
        if (property2 != null) {
            this.fontConfigFile = new File(property2);
        }
        else {
            this.fontConfigFile = this.findFontConfigFile(this.javaLib);
        }
        return;
    }
    
    private void readFontConfigFile(File file) {
        this.getInstalledFallbackFonts(this.javaLib);
        if (file != null) {
            try {
                FileInputStream fileInputStream = new FileInputStream(file.getPath());
                if (FontConfiguration.isProperties) {
                    loadProperties(fileInputStream);
                }
                else {
                    loadBinary(fileInputStream);
                }
                fileInputStream.close();
                if (FontUtilities.debugFonts()) {
                    FontConfiguration.logger.config("Read logical font configuration from " + file);
                }
            }
            catch (IOException ex) {
                if (FontUtilities.debugFonts()) {
                    FontConfiguration.logger.config("Failed to read logical font configuration from " + file);
                }
            }
        }
        String version = this.getVersion();
        if (!"1".equals(version) && FontUtilities.debugFonts()) {
            FontConfiguration.logger.config("Unsupported fontconfig version: " + version);
        }
    }
    
    protected void getInstalledFallbackFonts(String str) {
        String string = str + File.separator + "fonts" + File.separator + "fallback";
        File file = new File(string);
        if (file.exists() && file.isDirectory()) {
            String[] list = file.list(this.fontManager.getTrueTypeFilter());
            String[] list2 = file.list(this.fontManager.getType1Filter());
            int n = (list == null) ? 0 : list.length;
            int n2 = (list2 == null) ? 0 : list2.length;
            int n3 = n + n2;
            if (n + n2 == 0) {
                return;
            }
            FontConfiguration.installedFallbackFontFiles = new String[n3];
            for (int i = 0; i < n; ++i) {
                FontConfiguration.installedFallbackFontFiles[i] = file + File.separator + list[i];
            }
            for (int j = 0; j < n2; ++j) {
                FontConfiguration.installedFallbackFontFiles[j + n] = file + File.separator + list2[j];
            }
            this.fontManager.registerFontsInDir(string);
        }
    }
    
    private File findImpl(String s) {
        File file = new File(s + ".properties");
        if (file.canRead()) {
            FontConfiguration.isProperties = true;
            return file;
        }
        File file2 = new File(s + ".bfc");
        if (file2.canRead()) {
            FontConfiguration.isProperties = false;
            return file2;
        }
        return null;
    }
    
    private File findFontConfigFile(String str) {
        String string = str + File.separator + "fontconfig";
        String substring = null;
        if (FontConfiguration.osVersion != null && FontConfiguration.osName != null) {
            File impl = this.findImpl(string + "." + FontConfiguration.osName + "." + FontConfiguration.osVersion);
            if (impl != null) {
                return impl;
            }
            if (FontConfiguration.osVersion.indexOf(".") != -1) {
                substring = FontConfiguration.osVersion.substring(0, FontConfiguration.osVersion.indexOf("."));
                File impl2 = this.findImpl(string + "." + FontConfiguration.osName + "." + substring);
                if (impl2 != null) {
                    return impl2;
                }
            }
        }
        if (FontConfiguration.osName != null) {
            File impl3 = this.findImpl(string + "." + FontConfiguration.osName);
            if (impl3 != null) {
                return impl3;
            }
        }
        if (FontConfiguration.osVersion != null) {
            File impl4 = this.findImpl(string + "." + FontConfiguration.osVersion);
            if (impl4 != null) {
                return impl4;
            }
            if (substring != null) {
                File impl5 = this.findImpl(string + "." + substring);
                if (impl5 != null) {
                    return impl5;
                }
            }
        }
        this.foundOsSpecificFile = false;
        File impl6 = this.findImpl(string);
        if (impl6 != null) {
            return impl6;
        }
        return null;
    }
    
    public static void loadBinary(InputStream in) throws IOException {
        DataInputStream dataInputStream = new DataInputStream(in);
        FontConfiguration.head = readShortTable(dataInputStream, 20);
        int[] array = new int[14];
        for (int i = 0; i < 14; ++i) {
            array[i] = FontConfiguration.head[i + 1] - FontConfiguration.head[i];
        }
        FontConfiguration.table_scriptIDs = readShortTable(dataInputStream, array[0]);
        FontConfiguration.table_scriptFonts = readShortTable(dataInputStream, array[1]);
        FontConfiguration.table_elcIDs = readShortTable(dataInputStream, array[2]);
        FontConfiguration.table_sequences = readShortTable(dataInputStream, array[3]);
        FontConfiguration.table_fontfileNameIDs = readShortTable(dataInputStream, array[4]);
        FontConfiguration.table_componentFontNameIDs = readShortTable(dataInputStream, array[5]);
        FontConfiguration.table_filenames = readShortTable(dataInputStream, array[6]);
        FontConfiguration.table_awtfontpaths = readShortTable(dataInputStream, array[7]);
        FontConfiguration.table_exclusions = readShortTable(dataInputStream, array[8]);
        FontConfiguration.table_proportionals = readShortTable(dataInputStream, array[9]);
        FontConfiguration.table_scriptFontsMotif = readShortTable(dataInputStream, array[10]);
        FontConfiguration.table_alphabeticSuffix = readShortTable(dataInputStream, array[11]);
        FontConfiguration.table_stringIDs = readShortTable(dataInputStream, array[12]);
        FontConfiguration.stringCache = new String[FontConfiguration.table_stringIDs.length + 1];
        int n = array[13];
        byte[] b = new byte[n * 2];
        FontConfiguration.table_stringTable = new char[n];
        dataInputStream.read(b);
        for (int j = 0, n2 = 0; j < n; FontConfiguration.table_stringTable[j++] = (char)(b[n2++] << 8 | (b[n2++] & 0xFF))) {}
        if (FontConfiguration.verbose) {
            dump();
        }
    }
    
    public static void saveBinary(OutputStream out) throws IOException {
        sanityCheck();
        DataOutputStream dataOutputStream = new DataOutputStream(out);
        writeShortTable(dataOutputStream, FontConfiguration.head);
        writeShortTable(dataOutputStream, FontConfiguration.table_scriptIDs);
        writeShortTable(dataOutputStream, FontConfiguration.table_scriptFonts);
        writeShortTable(dataOutputStream, FontConfiguration.table_elcIDs);
        writeShortTable(dataOutputStream, FontConfiguration.table_sequences);
        writeShortTable(dataOutputStream, FontConfiguration.table_fontfileNameIDs);
        writeShortTable(dataOutputStream, FontConfiguration.table_componentFontNameIDs);
        writeShortTable(dataOutputStream, FontConfiguration.table_filenames);
        writeShortTable(dataOutputStream, FontConfiguration.table_awtfontpaths);
        writeShortTable(dataOutputStream, FontConfiguration.table_exclusions);
        writeShortTable(dataOutputStream, FontConfiguration.table_proportionals);
        writeShortTable(dataOutputStream, FontConfiguration.table_scriptFontsMotif);
        writeShortTable(dataOutputStream, FontConfiguration.table_alphabeticSuffix);
        writeShortTable(dataOutputStream, FontConfiguration.table_stringIDs);
        dataOutputStream.writeChars(new String(FontConfiguration.table_stringTable));
        out.close();
        if (FontConfiguration.verbose) {
            dump();
        }
    }
    
    public static void loadProperties(InputStream inputStream) throws IOException {
        FontConfiguration.stringIDNum = 1;
        FontConfiguration.stringIDs = new short[1000];
        FontConfiguration.stringTable = new StringBuilder(4096);
        if (FontConfiguration.verbose && FontConfiguration.logger == null) {
            FontConfiguration.logger = PlatformLogger.getLogger("sun.awt.FontConfiguration");
        }
        new FontConfiguration.PropertiesHandler().load(inputStream);
        FontConfiguration.stringIDs = null;
        FontConfiguration.stringTable = null;
    }
    
    private void initFontConfig() {
        this.initLocale = FontConfiguration.startupLocale;
        this.initEncoding = FontConfiguration.encoding;
        if (this.preferLocaleFonts && !willReorderForStartupLocale()) {
            this.preferLocaleFonts = false;
        }
        this.initELC = this.getInitELC();
        this.initAllComponentFonts();
    }
    
    private short getInitELC() {
        if (this.initELC != -1) {
            return this.initELC;
        }
        HashMap<String, Integer> hashMap = new HashMap();
        for (int i = 0; i < FontConfiguration.table_elcIDs.length; ++i) {
            hashMap.put(getString(FontConfiguration.table_elcIDs[i]), Integer.valueOf(i));
        }
        String language = this.initLocale.getLanguage();
        String key;
        if (hashMap.containsKey(key = this.initEncoding + "." + language + "." + this.initLocale.getCountry()) || hashMap.containsKey(key = this.initEncoding + "." + language) || hashMap.containsKey(key = this.initEncoding)) {
            this.initELC = ((Integer)hashMap.get(key)).shortValue();
        }
        else {
            this.initELC = ((Integer)hashMap.get("NULL.NULL.NULL")).shortValue();
        }
        int j = 0;
        while (j < FontConfiguration.table_alphabeticSuffix.length) {
            if (this.initELC == FontConfiguration.table_alphabeticSuffix[j]) {
                this.alphabeticSuffix = getString(FontConfiguration.table_alphabeticSuffix[j + 1]);
                return this.initELC;
            }
            j += 2;
        }
        return this.initELC;
    }
    
    private void initAllComponentFonts() {
        short[] fallbackScripts = getFallbackScripts();
        for (int i = 0; i < 5; ++i) {
            short[] coreScripts = this.getCoreScripts(i);
            this.compCoreNum[i] = coreScripts.length;
            int[][] array = new int[coreScripts.length][];
            for (int j = 0; j < coreScripts.length; ++j) {
                array[j] = getExclusionRanges(coreScripts[j]);
            }
            this.compExclusions[i] = array;
            short[] array2;
            for (int k = 0; k < 4; this.compFontNameIDs[i][k] = array2, ++k) {
                array2 = new short[coreScripts.length + fallbackScripts.length];
                int l;
                for (l = 0; l < coreScripts.length; ++l) {
                    array2[l] = getComponentFontID(coreScripts[l], i, k);
                    if (this.preferLocaleFonts && FontConfiguration.localeMap != null && this.fontManager.usingAlternateFontforJALocales()) {
                        array2[l] = this.remapLocaleMap(i, k, coreScripts[l], array2[l]);
                    }
                    if (this.preferPropFonts) {
                        array2[l] = this.remapProportional(i, array2[l]);
                    }
                    this.coreFontNameIDs.add(Short.valueOf(array2[l]));
                }
                for (int n = 0; n < fallbackScripts.length; ++n) {
                    int s = getComponentFontID(fallbackScripts[n], i, k);
                    if (this.preferLocaleFonts && FontConfiguration.localeMap != null && this.fontManager.usingAlternateFontforJALocales()) {
                        s = this.remapLocaleMap(i, k, fallbackScripts[n], (short)s);
                    }
                    if (this.preferPropFonts) {
                        s = this.remapProportional(i, (short)s);
                    }
                    if (!contains(array2, (short)s, l)) {
                        this.fallbackFontNameIDs.add(Short.valueOf((short)s));
                        array2[l++] = s;
                    }
                }
                if (l < array2.length) {
                    short[] array3 = new short[l];
                    System.arraycopy(array2, 0, array3, 0, l);
                    array2 = array3;
                }
            }
        }
    }
    
    private short remapLocaleMap(int n, int n2, short n3, short n4) {
        String string = getString(FontConfiguration.table_scriptIDs[n3]);
        String s = (String)FontConfiguration.localeMap.get(string);
        if (s == null) {
            s = (String)FontConfiguration.localeMap.get(FontConfiguration.fontNames[n] + "." + FontConfiguration.styleNames[n2] + "." + string);
        }
        if (s == null) {
            return n4;
        }
        int i = 0;
        while (i < FontConfiguration.table_componentFontNameIDs.length) {
            if (s.equalsIgnoreCase(getString(FontConfiguration.table_componentFontNameIDs[i]))) {
                n4 = (short)i;
                break;
            }
            ++i;
        }
        return n4;
    }
    
    public static boolean hasMonoToPropMap() {
        return FontConfiguration.table_proportionals != null && FontConfiguration.table_proportionals.length != 0;
    }
    
    private short remapProportional(int n, short n2) {
        if (this.preferPropFonts && FontConfiguration.table_proportionals.length != 0 && n != 2 && n != 4) {
            int i = 0;
            while (i < FontConfiguration.table_proportionals.length) {
                if (FontConfiguration.table_proportionals[i] == n2) {
                    return FontConfiguration.table_proportionals[i + 1];
                }
                i += 2;
            }
        }
        return n2;
    }
    
    public static boolean isLogicalFontFamilyName(String s) {
        return isLogicalFontFamilyNameLC(s.toLowerCase(Locale.ENGLISH));
    }
    
    public static boolean isLogicalFontFamilyNameLC(String s) {
        int i = 0;
        while (i < FontConfiguration.fontNames.length) {
            if (s.equals(FontConfiguration.fontNames[i])) {
                return true;
            }
            ++i;
        }
        return false;
    }
    
    private static boolean isLogicalFontStyleName(String s) {
        int i = 0;
        while (i < FontConfiguration.styleNames.length) {
            if (s.equals(FontConfiguration.styleNames[i])) {
                return true;
            }
            ++i;
        }
        return false;
    }
    
    public static boolean isLogicalFontFaceName(String s) {
        return isLogicalFontFaceNameLC(s.toLowerCase(Locale.ENGLISH));
    }
    
    public static boolean isLogicalFontFaceNameLC(String s) {
        int index = s.indexOf(46);
        if (index >= 0) {
            String substring = s.substring(0, index);
            String substring2 = s.substring(index + 1);
            return isLogicalFontFamilyName(substring) && isLogicalFontStyleName(substring2);
        }
        return isLogicalFontFamilyName(s);
    }
    
    protected static int getFontIndex(String s) {
        return getArrayIndex(FontConfiguration.fontNames, s);
    }
    
    protected static int getStyleIndex(String s) {
        return getArrayIndex(FontConfiguration.styleNames, s);
    }
    
    private static int getArrayIndex(String[] array, String s) {
        int i = 0;
        while (i < array.length) {
            if (s.equals(array[i])) {
                return i;
            }
            ++i;
        }
        assert false;
        return 0;
    }
    
    protected static int getStyleIndex(int n) {
        switch (n) {
            case 0: {
                return 0;
            }
            case 1: {
                return 1;
            }
            case 2: {
                return 2;
            }
            case 3: {
                return 3;
            }
            default: {
                return 0;
            }
        }
    }
    
    protected static String getFontName(int n) {
        return FontConfiguration.fontNames[n];
    }
    
    protected static String getStyleName(int n) {
        return FontConfiguration.styleNames[n];
    }
    
    public static String getLogicalFontFaceName(String s, int n) {
        assert !(!isLogicalFontFamilyName(s));
        return s.toLowerCase(Locale.ENGLISH) + "." + getStyleString(n);
    }
    
    public static String getStyleString(int n) {
        return getStyleName(getStyleIndex(n));
    }
    
    public abstract String getFallbackFamilyName(String p0, String p1);
    
    protected String getCompatibilityFamilyName(String lowerCase) {
        lowerCase = lowerCase.toLowerCase(Locale.ENGLISH);
        if (lowerCase.equals("timesroman")) {
            return "serif";
        }
        if (lowerCase.equals("helvetica")) {
            return "sansserif";
        }
        if (lowerCase.equals("courier")) {
            return "monospaced";
        }
        return null;
    }
    
    protected String mapFileName(String s) {
        return s;
    }
    
    protected abstract void initReorderMap();
    
    private void shuffle(String[] array, int i, int n) {
        if (n >= i) {
            return;
        }
        String s = array[i];
        for (; i > n; --i) {
            array[i] = array[i - 1];
        }
        array[n] = s;
        return;
    }
    
    public static boolean willReorderForStartupLocale() {
        return getReorderSequence() != null;
    }
    
    private static Object getReorderSequence() {
        if (FontConfiguration.fontConfig.reorderMap == null) {
            FontConfiguration.fontConfig.initReorderMap();
        }
        HashMap<Object, Object> reorderMap = FontConfiguration.fontConfig.reorderMap;
        String language = FontConfiguration.startupLocale.getLanguage();
        Object o = reorderMap.get(FontConfiguration.encoding + "." + language + "." + FontConfiguration.startupLocale.getCountry());
        if (o == null) {
            o = reorderMap.get(FontConfiguration.encoding + "." + language);
        }
        if (o == null) {
            o = reorderMap.get(FontConfiguration.encoding);
            return o;
        }
        return o;
    }
    
    private void reorderSequenceForLocale(String[] array) {
        Object reorderSequence = getReorderSequence();
        if ((reorderSequence instanceof String)) {
            int k = 0;
            while (k < array.length) {
                if (array[k].equals(reorderSequence)) {
                    this.shuffle(array, k, 0);
                    return;
                }
                ++k;
            }
        }
        else if ((reorderSequence instanceof String[])) {
            String[] array2 = (String[])(String[])reorderSequence;
            for (int i = 0; i < array2.length; ++i) {
                for (int j = 0; j < array.length; ++j) {
                    if (array[j].equals(array2[i])) {
                        this.shuffle(array, j, i);
                    }
                }
            }
        }
    }
    
    private static Vector splitSequence(String s) {
        Vector<String> vector = new Vector();
        int beginIndex;
        int index;
        for (beginIndex = 0; (index = s.indexOf(44, beginIndex)) >= 0; beginIndex = index + 1) {
            vector.add(s.substring(beginIndex, index));
        }
        if (s.length() > beginIndex) {
            vector.add(s.substring(beginIndex, s.length()));
            return vector;
        }
        return vector;
    }
    
    protected String[] split(String s) {
        return (String[])(String[])splitSequence(s).toArray(new String[0]);
    }
    
    public FontDescriptor[] getFontDescriptors(String lowerCase, int n) {
        assert !(!isLogicalFontFamilyName(lowerCase));
        lowerCase = lowerCase.toLowerCase(Locale.ENGLISH);
        return this.getFontDescriptors(getFontIndex(lowerCase), getStyleIndex(n));
    }
    
    private FontDescriptor[] getFontDescriptors(int n, int n2) {
        FontDescriptor[] buildFontDescriptors = this.fontDescriptors[n][n2];
        if (buildFontDescriptors == null) {
            buildFontDescriptors = this.buildFontDescriptors(n, n2);
            this.fontDescriptors[n][n2] = buildFontDescriptors;
            return buildFontDescriptors;
        }
        return buildFontDescriptors;
    }
    
    protected FontDescriptor[] buildFontDescriptors(int n, int n2) {
        String s = FontConfiguration.fontNames[n];
        String s2 = FontConfiguration.styleNames[n2];
        short[] coreScripts = this.getCoreScripts(n);
        short[] array = this.compFontNameIDs[n][n2];
        String[] array2 = new String[coreScripts.length];
        String[] array3 = new String[coreScripts.length];
        for (int i = 0; i < array2.length; ++i) {
            array3[i] = getComponentFontName(array[i]);
            array2[i] = getScriptName(coreScripts[i]);
            if (this.alphabeticSuffix != null && "alphabetic".equals(array2[i])) {
                array2[i] = array2[i] + "/" + this.alphabeticSuffix;
            }
        }
        int[][] array4 = this.compExclusions[n];
        FontDescriptor[] array5 = new FontDescriptor[array3.length];
        String awtFontName;
        String encoding;
        for (int j = 0; j < array3.length; array5[j] = new FontDescriptor(awtFontName, this.getFontCharsetEncoder(encoding.trim(), awtFontName), array4[j]), ++j) {
            awtFontName = this.makeAWTFontName(array3[j], array2[j]);
            encoding = this.getEncoding(array3[j], array2[j]);
            if (encoding == null) {
                encoding = "default";
            }
        }
        return array5;
    }
    
    protected String makeAWTFontName(String s, String s2) {
        return s;
    }
    
    protected abstract String getEncoding(String p0, String p1);
    
    private CharsetEncoder getFontCharsetEncoder(final String key, String s) {
        Charset charset;
        if (key.equals("default")) {
            charset = (Charset)this.charsetRegistry.get(s);
        }
        else {
            charset = (Charset)this.charsetRegistry.get(key);
        }
        if (charset != null) {
            return charset.newEncoder();
        }
        if (!key.startsWith("sun.awt.") && !key.equals("default")) {
            charset = Charset.forName(key);
        }
        else {
            Class<Object> clazz = (Class)AccessController.doPrivileged(new PrivilegedAction() {
                @Override
                public Object run() {
                    try {
                        return Class.forName(key, true, ClassLoader.getSystemClassLoader());
                    }
                    catch (ClassNotFoundException ex) {
                        return null;
                    }
                }
            });
            if (clazz != null) {
                try {
                    charset = (Charset)clazz.newInstance();
                }
                catch (Exception ex) {}
            }
        }
        if (charset == null) {
            charset = this.getDefaultFontCharset(s);
        }
        if (key.equals("default")) {
            this.charsetRegistry.put(s, charset);
        }
        else {
            this.charsetRegistry.put(key, charset);
        }
        return charset.newEncoder();
    }
    
    protected abstract Charset getDefaultFontCharset(String p0);
    
    public HashSet<String> getAWTFontPathSet() {
        return null;
    }
    
    public CompositeFontDescriptor[] get2DCompositeFontInfo() {
        CompositeFontDescriptor[] array = new CompositeFontDescriptor[20];
        String defaultFontFile = this.fontManager.getDefaultFontFile();
        String defaultFontFaceName = this.fontManager.getDefaultFontFaceName();
        for (int i = 0; i < 5; ++i) {
            String str = FontConfiguration.publicFontNames[i];
            int[][] array2 = this.compExclusions[i];
            int n = 0;
            for (int j = 0; j < array2.length; ++j) {
                n += array2[j].length;
            }
            int[] exclusionRanges = new int[n];
            int[] exclusionRangeLimits = new int[array2.length];
            for (int n2 = 0, k = 0; k < array2.length; exclusionRangeLimits[k] = n2, ++k) {
                int[] array3 = array2[k];
                for (int l = 0; l < array3.length; exclusionRanges[n2++] = array3[l++], exclusionRanges[n2++] = array3[l++]) {
                    int n3 = array3[l];
                }
            }
            String string;
            String[] componentFaceNames;
            String[] componentFileNames;
            for (int n4 = 0; n4 < 4; array[i * 4 + n4] = new CompositeFontDescriptor(string, this.compCoreNum[i], componentFaceNames, componentFileNames, exclusionRanges, exclusionRangeLimits), ++n4) {
                int length = this.compFontNameIDs[i][n4].length;
                int n5 = 0;
                if (FontConfiguration.installedFallbackFontFiles != null) {
                    length += FontConfiguration.installedFallbackFontFiles.length;
                }
                string = str + "." + FontConfiguration.styleNames[n4];
                componentFaceNames = new String[length];
                componentFileNames = new String[length];
                int n6;
                for (n6 = 0; n6 < this.compFontNameIDs[i][n4].length; ++n6) {
                    int n7 = this.compFontNameIDs[i][n4][n6];
                    int componentFileID = getComponentFileID((short)n7);
                    componentFaceNames[n6] = this.getFaceNameFromComponentFontName(getComponentFontName((short)n7));
                    componentFileNames[n6] = this.mapFileName(getComponentFileName((short)componentFileID));
                    if (componentFileNames[n6] == null || this.needToSearchForFile(componentFileNames[n6])) {
                        componentFileNames[n6] = this.getFileNameFromComponentFontName(getComponentFontName((short)n7));
                    }
                    if (n5 == 0 && defaultFontFile.equals(componentFileNames[n6])) {
                        n5 = 1;
                    }
                }
                if (n5 == 0) {
                    int length2 = 0;
                    if (FontConfiguration.installedFallbackFontFiles != null) {
                        length2 = FontConfiguration.installedFallbackFontFiles.length;
                    }
                    if (n6 + length2 == length) {
                        String[] array4 = new String[length + 1];
                        System.arraycopy(componentFaceNames, 0, array4, 0, n6);
                        componentFaceNames = array4;
                        String[] array5 = new String[length + 1];
                        System.arraycopy(componentFileNames, 0, array5, 0, n6);
                        componentFileNames = array5;
                    }
                    componentFaceNames[n6] = defaultFontFaceName;
                    componentFileNames[n6] = defaultFontFile;
                    ++n6;
                }
                if (FontConfiguration.installedFallbackFontFiles != null) {
                    for (int n8 = 0; n8 < FontConfiguration.installedFallbackFontFiles.length; ++n8) {
                        componentFaceNames[n6] = null;
                        componentFileNames[n6] = FontConfiguration.installedFallbackFontFiles[n8];
                        ++n6;
                    }
                }
                if (n6 < length) {
                    String[] array6 = new String[n6];
                    System.arraycopy(componentFaceNames, 0, array6, 0, n6);
                    componentFaceNames = array6;
                    String[] array7 = new String[n6];
                    System.arraycopy(componentFileNames, 0, array7, 0, n6);
                    componentFileNames = array7;
                }
                if (n6 != exclusionRangeLimits.length) {
                    int length3 = exclusionRangeLimits.length;
                    exclusionRangeLimits = new int[n6];
                    System.arraycopy(exclusionRangeLimits, 0, exclusionRangeLimits, 0, length3);
                    for (; length3 < n6; ++length3) {
                        exclusionRangeLimits[length3] = exclusionRanges.length;
                    }
                }
            }
        }
        return array;
    }
    
    protected abstract String getFaceNameFromComponentFontName(String p0);
    
    protected abstract String getFileNameFromComponentFontName(String p0);
    
    public boolean needToSearchForFile(String str) {
        if (!FontUtilities.isLinux) {
            return false;
        }
        if (this.existsMap == null) {
            this.existsMap = new HashMap();
        }
        Boolean value = (Boolean)this.existsMap.get(str);
        if (value == null) {
            this.getNumberCoreFonts();
            if (!this.coreFontFileNames.contains(str)) {
                value = Boolean.TRUE;
            }
            else {
                value = Boolean.valueOf(new File(str).exists());
                this.existsMap.put(str, value);
                if (FontUtilities.debugFonts() && value == Boolean.FALSE) {
                    FontConfiguration.logger.warning("Couldn't locate font file " + str);
                }
            }
        }
        return value == Boolean.FALSE;
    }
    
    public int getNumberCoreFonts() {
        if (this.numCoreFonts == -1) {
            this.numCoreFonts = this.coreFontNameIDs.size();
            Short[] array = new Short[0];
            Short[] array2 = (Short[])this.coreFontNameIDs.toArray(array);
            Short[] array3 = (Short[])this.fallbackFontNameIDs.toArray(array);
            int n = 0;
            for (int i = 0; i < array3.length; ++i) {
                if (this.coreFontNameIDs.contains(array3[i])) {
                    array3[i] = null;
                }
                else {
                    ++n;
                }
            }
            this.componentFonts = new String[this.numCoreFonts + n];
            int j;
            for (j = 0; j < array2.length; ++j) {
                int shortValue = (short)array2[j];
                int componentFileID = getComponentFileID((short)shortValue);
                this.componentFonts[j] = getComponentFontName((short)shortValue);
                String componentFileName = getComponentFileName((short)componentFileID);
                if (componentFileName != null) {
                    this.coreFontFileNames.add(componentFileName);
                }
                this.filenamesMap.put(this.componentFonts[j], this.mapFileName(componentFileName));
            }
            for (int k = 0; k < array3.length; ++k) {
                if (array3[k] != null) {
                    int shortValue2 = (short)array3[k];
                    int componentFileID2 = getComponentFileID((short)shortValue2);
                    this.componentFonts[j] = getComponentFontName((short)shortValue2);
                    this.filenamesMap.put(this.componentFonts[j], this.mapFileName(getComponentFileName((short)componentFileID2)));
                    ++j;
                }
            }
        }
        return this.numCoreFonts;
    }
    
    public String[] getPlatformFontNames() {
        if (this.numCoreFonts == -1) {
            this.getNumberCoreFonts();
        }
        return this.componentFonts;
    }
    
    public String getFileNameFromPlatformName(String key) {
        return (String)this.filenamesMap.get(key);
    }
    
    public String getExtraFontPath() {
        return getString(FontConfiguration.head[16]);
    }
    
    public String getVersion() {
        return getString(FontConfiguration.head[17]);
    }
    
    protected static FontConfiguration getFontConfiguration() {
        return FontConfiguration.fontConfig;
    }
    
    protected void setFontConfiguration() {
        FontConfiguration.fontConfig = this;
    }
    
    private static void sanityCheck() {
        int i = 0;
        String anObject = (String)AccessController.doPrivileged(new PrivilegedAction() {
            @Override
            public Object run() {
                return System.getProperty("os.name");
            }
        });
        for (int j = 1; j < FontConfiguration.table_filenames.length; ++j) {
            if (FontConfiguration.table_filenames[j] == -1) {
                if (anObject.contains("Windows")) {
                    System.err.println("\n Error: <filename." + getString(FontConfiguration.table_componentFontNameIDs[j]) + "> entry is missing!!!");
                    ++i;
                }
                else if (FontConfiguration.verbose && !isEmpty(FontConfiguration.table_filenames)) {
                    System.err.println("\n Note: 'filename' entry is undefined for \"" + getString(FontConfiguration.table_componentFontNameIDs[j]) + "\"");
                }
            }
        }
        for (int k = 0; k < FontConfiguration.table_scriptIDs.length; ++k) {
            int n = FontConfiguration.table_scriptFonts[k];
            if (n == 0) {
                System.out.println("\n Error: <allfonts." + getString(FontConfiguration.table_scriptIDs[k]) + "> entry is missing!!!");
                ++i;
            }
            else if (n < 0) {
                int n2 = (short)(-n);
                for (int l = 0; l < 5; ++l) {
                    for (int n3 = 0; n3 < 4; ++n3) {
                        if (FontConfiguration.table_scriptFonts[n2 + (l * 4 + n3)] == 0) {
                            System.err.println("\n Error: <" + getFontName(l) + "." + getStyleName(n3) + "." + getString(FontConfiguration.table_scriptIDs[k]) + "> entry is missing!!!");
                            ++i;
                        }
                    }
                }
            }
        }
        if ("SunOS".equals(anObject)) {
            for (int n4 = 0; n4 < FontConfiguration.table_awtfontpaths.length; ++n4) {
                if (FontConfiguration.table_awtfontpaths[n4] == 0) {
                    String string = getString(FontConfiguration.table_scriptIDs[n4]);
                    if (!string.contains("lucida") && !string.contains("dingbats") && !string.contains("symbol")) {
                        System.err.println("\nError: <awtfontpath." + string + "> entry is missing!!!");
                        ++i;
                    }
                }
            }
        }
        if (i != 0) {
            System.err.println("!!THERE ARE " + i + " ERROR(S) IN THE FONTCONFIG FILE, PLEASE CHECK ITS CONTENT!!\n");
            System.exit(1);
        }
    }
    
    private static boolean isEmpty(short[] array) {
        int length = array.length;
        int i = 0;
        while (i < length) {
            if (array[i] != -1) {
                return false;
            }
            ++i;
        }
        return true;
    }
    
    private static void dump() {
        System.out.println("\n----Head Table------------");
        for (int i = 0; i < 20; ++i) {
            System.out.println("  " + i + " : " + (int)FontConfiguration.head[i]);
        }
        System.out.println("\n----scriptIDs-------------");
        printTable(FontConfiguration.table_scriptIDs, 0);
        System.out.println("\n----scriptFonts----------------");
        for (int j = 0; j < FontConfiguration.table_scriptIDs.length; ++j) {
            int n = FontConfiguration.table_scriptFonts[j];
            if (n >= 0) {
                System.out.println("  allfonts." + getString(FontConfiguration.table_scriptIDs[j]) + "=" + getString(FontConfiguration.table_componentFontNameIDs[n]));
            }
        }
        for (int k = 0; k < FontConfiguration.table_scriptIDs.length; ++k) {
            int n2 = FontConfiguration.table_scriptFonts[k];
            if (n2 < 0) {
                int n3 = (short)(-n2);
                for (int l = 0; l < 5; ++l) {
                    for (int n4 = 0; n4 < 4; ++n4) {
                        System.out.println("  " + getFontName(l) + "." + getStyleName(n4) + "." + getString(FontConfiguration.table_scriptIDs[k]) + "=" + getString(FontConfiguration.table_componentFontNameIDs[FontConfiguration.table_scriptFonts[n3 + (l * 4 + n4)]]));
                    }
                }
            }
        }
        System.out.println("\n----elcIDs----------------");
        printTable(FontConfiguration.table_elcIDs, 0);
        System.out.println("\n----sequences-------------");
        for (int m = 0; m < FontConfiguration.table_elcIDs.length; ++m) {
            System.out.println("  " + m + "/" + getString(FontConfiguration.table_elcIDs[m]));
            short[] shortArray = getShortArray(FontConfiguration.table_sequences[m * 5 + 0]);
            for (int n5 = 0; n5 < shortArray.length; ++n5) {
                System.out.println("     " + getString(FontConfiguration.table_scriptIDs[shortArray[n5]]));
            }
        }
        System.out.println("\n----fontfileNameIDs-------");
        printTable(FontConfiguration.table_fontfileNameIDs, 0);
        System.out.println("\n----componentFontNameIDs--");
        printTable(FontConfiguration.table_componentFontNameIDs, 1);
        System.out.println("\n----filenames-------------");
        for (int n6 = 0; n6 < FontConfiguration.table_filenames.length; ++n6) {
            if (FontConfiguration.table_filenames[n6] == -1) {
                System.out.println("  " + n6 + " : null");
            }
            else {
                System.out.println("  " + n6 + " : " + getString(FontConfiguration.table_fontfileNameIDs[FontConfiguration.table_filenames[n6]]));
            }
        }
        System.out.println("\n----awtfontpaths---------");
        for (int n7 = 0; n7 < FontConfiguration.table_awtfontpaths.length; ++n7) {
            System.out.println("  " + getString(FontConfiguration.table_scriptIDs[n7]) + " : " + getString(FontConfiguration.table_awtfontpaths[n7]));
        }
        System.out.println("\n----proportionals--------");
        for (int n8 = 0; n8 < FontConfiguration.table_proportionals.length; ++n8) {
            System.out.println("  " + getString(FontConfiguration.table_componentFontNameIDs[FontConfiguration.table_proportionals[n8++]]) + " -> " + getString(FontConfiguration.table_componentFontNameIDs[FontConfiguration.table_proportionals[n8]]));
        }
        int n9 = 0;
        System.out.println("\n----alphabeticSuffix----");
        while (n9 < FontConfiguration.table_alphabeticSuffix.length) {
            System.out.println("    " + getString(FontConfiguration.table_elcIDs[FontConfiguration.table_alphabeticSuffix[n9++]]) + " -> " + getString(FontConfiguration.table_alphabeticSuffix[n9++]));
        }
        System.out.println("\n----String Table---------");
        System.out.println("    stringID:    Num =" + FontConfiguration.table_stringIDs.length);
        System.out.println("    stringTable: Size=" + FontConfiguration.table_stringTable.length * 2);
        System.out.println("\n----fallbackScriptIDs---");
        short[] shortArray2 = getShortArray(FontConfiguration.head[15]);
        for (int n10 = 0; n10 < shortArray2.length; ++n10) {
            System.out.println("  " + getString(FontConfiguration.table_scriptIDs[shortArray2[n10]]));
        }
        System.out.println("\n----appendedfontpath-----");
        System.out.println("  " + getString(FontConfiguration.head[16]));
        System.out.println("\n----Version--------------");
        System.out.println("  " + getString(FontConfiguration.head[17]));
    }
    
    protected static short getComponentFontID(short n, int n2, int n3) {
        int n4 = FontConfiguration.table_scriptFonts[n];
        if (n4 >= 0) {
            return n4;
        }
        return FontConfiguration.table_scriptFonts[-n4 + n2 * 4 + n3];
    }
    
    protected static short getComponentFontIDMotif(short n, int n2, int n3) {
        if (FontConfiguration.table_scriptFontsMotif.length == 0) {
            return 0;
        }
        int n4 = FontConfiguration.table_scriptFontsMotif[n];
        if (n4 >= 0) {
            return n4;
        }
        return FontConfiguration.table_scriptFontsMotif[-n4 + n2 * 4 + n3];
    }
    
    private static int[] getExclusionRanges(short n) {
        int n2 = FontConfiguration.table_exclusions[n];
        if (n2 == 0) {
            return FontConfiguration.EMPTY_INT_ARRAY;
        }
        char[] charArray = getString((short)n2).toCharArray();
        int[] array = new int[charArray.length / 2];
        int n3 = 0;
        for (int i = 0; i < array.length; ++i) {
            array[i] = (charArray[n3++] << 16) + (charArray[n3++] & '\uffff');
        }
        return array;
    }
    
    private static boolean contains(short[] array, short n, int n2) {
        int i = 0;
        while (i < n2) {
            if (array[i] == n) {
                return true;
            }
            ++i;
        }
        return false;
    }
    
    protected static String getComponentFontName(short n) {
        if (n < 0) {
            return null;
        }
        return getString(FontConfiguration.table_componentFontNameIDs[n]);
    }
    
    private static String getComponentFileName(short n) {
        if (n < 0) {
            return null;
        }
        return getString(FontConfiguration.table_fontfileNameIDs[n]);
    }
    
    private static short getComponentFileID(short n) {
        return FontConfiguration.table_filenames[n];
    }
    
    private static String getScriptName(short n) {
        return getString(FontConfiguration.table_scriptIDs[n]);
    }
    
    protected short[] getCoreScripts(int n) {
        short[] shortArray = getShortArray(FontConfiguration.table_sequences[this.getInitELC() * 5 + n]);
        if (this.preferLocaleFonts) {
            if (this.reorderScripts == null) {
                this.reorderScripts = new HashMap();
            }
            String[] array = new String[shortArray.length];
            for (int i = 0; i < array.length; ++i) {
                array[i] = getScriptName(shortArray[i]);
                this.reorderScripts.put(array[i], Short.valueOf(shortArray[i]));
            }
            this.reorderSequenceForLocale(array);
            for (int j = 0; j < array.length; ++j) {
                shortArray[j] = (short)this.reorderScripts.get(array[j]);
            }
            return shortArray;
        }
        else {
            return shortArray;
        }
    }
    
    private static short[] getFallbackScripts() {
        return getShortArray(FontConfiguration.head[15]);
    }
    
    private static void printTable(short[] array, int i) {
        for (; i < array.length; ++i) {
            System.out.println("  " + i + " : " + getString(array[i]));
        }
    }
    
    private static short[] readShortTable(DataInputStream dataInputStream, int n) throws IOException {
        if (n == 0) {
            return FontConfiguration.EMPTY_SHORT_ARRAY;
        }
        short[] array = new short[n];
        byte[] b = new byte[n * 2];
        dataInputStream.read(b);
        for (int i = 0, n2 = 0; i < n; array[i++] = (short)(b[n2++] << 8 | (b[n2++] & 0xFF))) {}
        return array;
    }
    
    private static void writeShortTable(DataOutputStream dataOutputStream, short[] array) throws IOException {
        for (int length = array.length, i = 0; i < length; ++i) {
            dataOutputStream.writeShort((int)array[i]);
        }
    }
    
    private static short[] toList(HashMap<String, Short> hashMap) {
        short[] a = new short[hashMap.size()];
        Arrays.fill(a, (short)(-1));
        for (Map.Entry entry : hashMap.entrySet()) {
            a[(short)entry.getValue()] = getStringID((String)entry.getKey());
        }
        return a;
    }
    
    protected static String getString(short n) {
        if (n == 0) {
            return null;
        }
        if (FontConfiguration.stringCache[n] == null) {
            FontConfiguration.stringCache[n] = new String(FontConfiguration.table_stringTable, (int)FontConfiguration.table_stringIDs[n], FontConfiguration.table_stringIDs[n + 1] - FontConfiguration.table_stringIDs[n]);
        }
        return FontConfiguration.stringCache[n];
    }
    
    private static short[] getShortArray(short n) {
        char[] charArray = getString(n).toCharArray();
        short[] array = new short[charArray.length];
        for (int i = 0; i < charArray.length; ++i) {
            array[i] = (short)(charArray[i] & '\uffff');
        }
        return array;
    }
    
    private static short getStringID(String str) {
        if (str == null) {
            return 0;
        }
        int n = (short)FontConfiguration.stringTable.length();
        FontConfiguration.stringTable.append(str);
        int n2 = (short)FontConfiguration.stringTable.length();
        FontConfiguration.stringIDs[FontConfiguration.stringIDNum] = n;
        FontConfiguration.stringIDs[FontConfiguration.stringIDNum + 1] = n2;
        ++FontConfiguration.stringIDNum;
        if (FontConfiguration.stringIDNum + 1 >= FontConfiguration.stringIDs.length) {
            short[] stringIDs = new short[FontConfiguration.stringIDNum + 1000];
            System.arraycopy(FontConfiguration.stringIDs, 0, stringIDs, 0, (int)FontConfiguration.stringIDNum);
            FontConfiguration.stringIDs = stringIDs;
        }
        return (short)(FontConfiguration.stringIDNum - 1);
    }
    
    private static short getShortArrayID(short[] array) {
        char[] value = new char[array.length];
        for (int i = 0; i < array.length; ++i) {
            value[i] = (char)array[i];
        }
        return getStringID(new String(value));
    }
    
    static short[] access$200(HashMap hashMap) {
        return toList(hashMap);
    }
    
    static short access$1000(String s) {
        return getStringID(s);
    }
    
    static short access$1500(short[] array) {
        return getShortArrayID(array);
    }
    
    static Vector access$2600(String s) {
        return splitSequence(s);
    }
    
    static {
        FontConfiguration.startupLocale = null;
        FontConfiguration.localeMap = null;
        FontConfiguration.isProperties = true;
        fontNames = new String[] { "serif", "sansserif", "monospaced", "dialog", "dialoginput" };
        publicFontNames = new String[] { "Serif", "SansSerif", "Monospaced", "Dialog", "DialogInput" };
        styleNames = new String[] { "plain", "bold", "italic", "bolditalic" };
        FontConfiguration.installedFallbackFontFiles = null;
        EMPTY_INT_ARRAY = new int[0];
        EMPTY_STRING_ARRAY = new String[0];
        EMPTY_SHORT_ARRAY = new short[0];
    }
    
    static class PropertiesHandler
    {
        private HashMap<String, Short> scriptIDs;
        private HashMap<String, Short> elcIDs;
        private HashMap<String, Short> componentFontNameIDs;
        private HashMap<String, Short> fontfileNameIDs;
        private HashMap<String, Integer> logicalFontIDs;
        private HashMap<String, Integer> fontStyleIDs;
        private HashMap<Short, Short> filenames;
        private HashMap<Short, short[]> sequences;
        private HashMap<Short, Short[]> scriptFonts;
        private HashMap<Short, Short> scriptAllfonts;
        private HashMap<Short, int[]> exclusions;
        private HashMap<Short, Short> awtfontpaths;
        private HashMap<Short, Short> proportionals;
        private HashMap<Short, Short> scriptAllfontsMotif;
        private HashMap<Short, Short[]> scriptFontsMotif;
        private HashMap<Short, Short> alphabeticSuffix;
        private short[] fallbackScriptIDs;
        private String version;
        private String appendedfontpath;
        
        public void load(InputStream inStream) throws IOException {
            this.initLogicalNameStyle();
            this.initHashMaps();
            new FontConfiguration.PropertiesHandler.FontProperties().load(inStream);
            this.initBinaryTable();
        }
        
        private void initBinaryTable() {
            FontConfiguration.head = new short[20];
            FontConfiguration.head[0] = 20;
            FontConfiguration.table_scriptIDs = FontConfiguration.access$200(this.scriptIDs);
            FontConfiguration.head[1] = (short)(FontConfiguration.head[0] + FontConfiguration.table_scriptIDs.length);
            FontConfiguration.table_scriptFonts = new short[FontConfiguration.table_scriptIDs.length + this.scriptFonts.size() * 20];
            for (Map.Entry entry : this.scriptAllfonts.entrySet()) {
                FontConfiguration.table_scriptFonts[(int)(Short)entry.getKey()] = (short)entry.getValue();
            }
            int length = FontConfiguration.table_scriptIDs.length;
            for (Map.Entry entry2 : this.scriptFonts.entrySet()) {
                FontConfiguration.table_scriptFonts[(int)(Short)entry2.getKey()] = (short)(-length);
                Short[] array = (Short[])entry2.getValue();
                for (int i = 0; i < 20; ++i) {
                    if (array[i] != null) {
                        FontConfiguration.table_scriptFonts[length++] = (short)array[i];
                    }
                    else {
                        FontConfiguration.table_scriptFonts[length++] = 0;
                    }
                }
            }
            FontConfiguration.head[2] = (short)(FontConfiguration.head[1] + FontConfiguration.table_scriptFonts.length);
            FontConfiguration.table_elcIDs = FontConfiguration.access$200(this.elcIDs);
            FontConfiguration.head[3] = (short)(FontConfiguration.head[2] + FontConfiguration.table_elcIDs.length);
            FontConfiguration.table_sequences = new short[this.elcIDs.size() * 5];
            for (Map.Entry entry3 : this.sequences.entrySet()) {
                int intValue = (int)(Short)entry3.getKey();
                short[] array2 = (short[])entry3.getValue();
                if (array2.length == 1) {
                    for (int k = 0; k < 5; ++k) {
                        FontConfiguration.table_sequences[intValue * 5 + k] = array2[0];
                    }
                }
                else {
                    for (int j = 0; j < 5; ++j) {
                        FontConfiguration.table_sequences[intValue * 5 + j] = array2[j];
                    }
                }
            }
            FontConfiguration.head[4] = (short)(FontConfiguration.head[3] + FontConfiguration.table_sequences.length);
            FontConfiguration.table_fontfileNameIDs = FontConfiguration.access$200(this.fontfileNameIDs);
            FontConfiguration.head[5] = (short)(FontConfiguration.head[4] + FontConfiguration.table_fontfileNameIDs.length);
            FontConfiguration.table_componentFontNameIDs = FontConfiguration.access$200(this.componentFontNameIDs);
            FontConfiguration.head[6] = (short)(FontConfiguration.head[5] + FontConfiguration.table_componentFontNameIDs.length);
            FontConfiguration.table_filenames = new short[FontConfiguration.table_componentFontNameIDs.length];
            Arrays.fill(FontConfiguration.table_filenames, (short)(-1));
            for (Map.Entry entry4 : this.filenames.entrySet()) {
                FontConfiguration.table_filenames[(short)entry4.getKey()] = (short)entry4.getValue();
            }
            FontConfiguration.head[7] = (short)(FontConfiguration.head[6] + FontConfiguration.table_filenames.length);
            FontConfiguration.table_awtfontpaths = new short[FontConfiguration.table_scriptIDs.length];
            for (Map.Entry entry5 : this.awtfontpaths.entrySet()) {
                FontConfiguration.table_awtfontpaths[(short)entry5.getKey()] = (short)entry5.getValue();
            }
            FontConfiguration.head[8] = (short)(FontConfiguration.head[7] + FontConfiguration.table_awtfontpaths.length);
            FontConfiguration.table_exclusions = new short[this.scriptIDs.size()];
            for (Map.Entry entry6 : this.exclusions.entrySet()) {
                int[] array3 = (int[])entry6.getValue();
                char[] value = new char[array3.length * 2];
                int n = 0;
                for (int l = 0; l < array3.length; ++l) {
                    value[n++] = (char)(array3[l] >> 16);
                    value[n++] = (char)(array3[l] & 0xFFFF);
                }
                FontConfiguration.table_exclusions[(short)entry6.getKey()] = FontConfiguration.access$1000(new String(value));
            }
            FontConfiguration.head[9] = (short)(FontConfiguration.head[8] + FontConfiguration.table_exclusions.length);
            FontConfiguration.table_proportionals = new short[this.proportionals.size() * 2];
            int n2 = 0;
            for (Map.Entry entry7 : this.proportionals.entrySet()) {
                FontConfiguration.table_proportionals[n2++] = (short)entry7.getKey();
                FontConfiguration.table_proportionals[n2++] = (short)entry7.getValue();
            }
            FontConfiguration.head[10] = (short)(FontConfiguration.head[9] + FontConfiguration.table_proportionals.length);
            if (this.scriptAllfontsMotif.size() != 0 || this.scriptFontsMotif.size() != 0) {
                FontConfiguration.table_scriptFontsMotif = new short[FontConfiguration.table_scriptIDs.length + this.scriptFontsMotif.size() * 20];
                for (Map.Entry entry8 : this.scriptAllfontsMotif.entrySet()) {
                    FontConfiguration.table_scriptFontsMotif[(int)(Short)entry8.getKey()] = (short)entry8.getValue();
                }
                int length2 = FontConfiguration.table_scriptIDs.length;
                for (Map.Entry entry9 : this.scriptFontsMotif.entrySet()) {
                    FontConfiguration.table_scriptFontsMotif[(int)(Short)entry9.getKey()] = (short)(-length2);
                    Short[] array4 = (Short[])entry9.getValue();
                    for (int n3 = 0; n3 < 20; ++n3) {
                        if (array4[n3] != null) {
                            FontConfiguration.table_scriptFontsMotif[length2++] = (short)array4[n3];
                        }
                        else {
                            FontConfiguration.table_scriptFontsMotif[length2++] = 0;
                        }
                    }
                }
            }
            else {
                FontConfiguration.table_scriptFontsMotif = FontConfiguration.EMPTY_SHORT_ARRAY;
            }
            FontConfiguration.head[11] = (short)(FontConfiguration.head[10] + FontConfiguration.table_scriptFontsMotif.length);
            FontConfiguration.table_alphabeticSuffix = new short[this.alphabeticSuffix.size() * 2];
            int n4 = 0;
            for (Map.Entry entry10 : this.alphabeticSuffix.entrySet()) {
                FontConfiguration.table_alphabeticSuffix[n4++] = (short)entry10.getKey();
                FontConfiguration.table_alphabeticSuffix[n4++] = (short)entry10.getValue();
            }
            FontConfiguration.head[15] = FontConfiguration.access$1500(this.fallbackScriptIDs);
            FontConfiguration.head[16] = FontConfiguration.access$1000(this.appendedfontpath);
            FontConfiguration.head[17] = FontConfiguration.access$1000(this.version);
            FontConfiguration.head[12] = (short)(FontConfiguration.head[11] + FontConfiguration.table_alphabeticSuffix.length);
            FontConfiguration.table_stringIDs = new short[FontConfiguration.stringIDNum + 1];
            System.arraycopy(FontConfiguration.stringIDs, 0, FontConfiguration.table_stringIDs, 0, FontConfiguration.stringIDNum + 1);
            FontConfiguration.head[13] = (short)(FontConfiguration.head[12] + FontConfiguration.stringIDNum + 1);
            FontConfiguration.table_stringTable = FontConfiguration.stringTable.toString().toCharArray();
            FontConfiguration.head[14] = (short)(FontConfiguration.head[13] + FontConfiguration.stringTable.length());
            FontConfiguration.stringCache = new String[FontConfiguration.table_stringIDs.length];
        }
        
        private void initLogicalNameStyle() {
            this.logicalFontIDs = new HashMap();
            this.fontStyleIDs = new HashMap();
            this.logicalFontIDs.put("serif", Integer.valueOf(0));
            this.logicalFontIDs.put("sansserif", Integer.valueOf(1));
            this.logicalFontIDs.put("monospaced", Integer.valueOf(2));
            this.logicalFontIDs.put("dialog", Integer.valueOf(3));
            this.logicalFontIDs.put("dialoginput", Integer.valueOf(4));
            this.fontStyleIDs.put("plain", Integer.valueOf(0));
            this.fontStyleIDs.put("bold", Integer.valueOf(1));
            this.fontStyleIDs.put("italic", Integer.valueOf(2));
            this.fontStyleIDs.put("bolditalic", Integer.valueOf(3));
        }
        
        private void initHashMaps() {
            this.scriptIDs = new HashMap();
            this.elcIDs = new HashMap();
            (this.componentFontNameIDs = new HashMap()).put("", Short.valueOf((short)0));
            this.fontfileNameIDs = new HashMap();
            this.filenames = new HashMap();
            this.sequences = new HashMap();
            this.scriptFonts = new HashMap();
            this.scriptAllfonts = new HashMap();
            this.exclusions = new HashMap();
            this.awtfontpaths = new HashMap();
            this.proportionals = new HashMap();
            this.scriptFontsMotif = new HashMap();
            this.scriptAllfontsMotif = new HashMap();
            this.alphabeticSuffix = new HashMap();
            this.fallbackScriptIDs = FontConfiguration.EMPTY_SHORT_ARRAY;
        }
        
        private int[] parseExclusions(String str, String s) {
            if (s == null) {
                return FontConfiguration.EMPTY_INT_ARRAY;
            }
            int n = 1;
            for (int index = 0; (index = s.indexOf(44, index)) != -1; ++index) {
                ++n;
            }
            int[] array = new int[n * 2];
            int n2 = 0;
            int int1;
            int int2;
            for (int i = 0; i < n * 2; array[i++] = int1, array[i++] = int2) {
                try {
                    int index2 = s.indexOf(45, n2);
                    String substring = s.substring(n2, index2);
                    int n3 = index2 + 1;
                    int endIndex = s.indexOf(44, n3);
                    if (endIndex == -1) {
                        endIndex = s.length();
                    }
                    String substring2 = s.substring(n3, endIndex);
                    n2 = endIndex + 1;
                    int length = substring.length();
                    int length2 = substring2.length();
                    if ((length != 4 && length != 6) || (length2 != 4 && length2 != 6)) {
                        throw new Exception();
                    }
                    int1 = Integer.parseInt(substring, 16);
                    int2 = Integer.parseInt(substring2, 16);
                    if (int1 > int2) {
                        throw new Exception();
                    }
                }
                catch (Exception ex) {
                    if (FontUtilities.debugFonts() && FontConfiguration.logger != null) {
                        FontConfiguration.logger.config("Failed parsing " + str + " property of font configuration.");
                    }
                    return FontConfiguration.EMPTY_INT_ARRAY;
                }
            }
            return array;
        }
        
        private Short getID(HashMap<String, Short> hashMap, String key) {
            Short n = (Short)hashMap.get(key);
            if (n == null) {
                hashMap.put(key, Short.valueOf((short)hashMap.size()));
                return (Short)hashMap.get(key);
            }
            return n;
        }
        
        private void parseProperty(String s, String s2) {
            if (s.startsWith("filename.")) {
                s = s.substring(9);
                if (!"MingLiU_HKSCS".equals(s)) {
                    s = s.replace('_', ' ');
                }
                this.filenames.put(this.getID(this.componentFontNameIDs, s), this.getID(this.fontfileNameIDs, s2));
            }
            else if (s.startsWith("exclusion.")) {
                s = s.substring(10);
                this.exclusions.put(this.getID(this.scriptIDs, s), this.parseExclusions(s, s2));
            }
            else if (s.startsWith("sequence.")) {
                s = s.substring(9);
                boolean n4 = false;
                boolean n5 = false;
                String[] array2 = (String[])(String[])FontConfiguration.access$2600(s2).toArray(FontConfiguration.EMPTY_STRING_ARRAY);
                short[] fallbackScriptIDs = new short[array2.length];
                for (int i = 0; i < array2.length; fallbackScriptIDs[i] = (short)this.getID(this.scriptIDs, array2[i]), ++i) {
                    if ("alphabetic/default".equals(array2[i])) {
                        array2[i] = "alphabetic";
                        n4 = true;
                    }
                    else if ("alphabetic/1252".equals(array2[i])) {
                        array2[i] = "alphabetic";
                        n5 = true;
                    }
                }
                int access$1500 = FontConfiguration.access$1500(fallbackScriptIDs);
                int index3 = s.indexOf(46);
                Short n6;
                if (index3 == -1) {
                    if ("fallback".equals(s)) {
                        this.fallbackScriptIDs = fallbackScriptIDs;
                        return;
                    }
                    if ("allfonts".equals(s)) {
                        n6 = this.getID(this.elcIDs, "NULL.NULL.NULL");
                    }
                    else {
                        if (FontConfiguration.logger != null) {
                            FontConfiguration.logger.config("Error sequence def: <sequence." + s + ">");
                        }
                        return;
                    }
                }
                else {
                    n6 = this.getID(this.elcIDs, s.substring(index3 + 1));
                    s = s.substring(0, index3);
                }
                short[] value;
                if ("allfonts".equals(s)) {
                    value = new short[] { access$1500 };
                }
                else {
                    value = (short[])this.sequences.get(n6);
                    if (value == null) {
                        value = new short[5];
                    }
                    Integer n7 = (Integer)this.logicalFontIDs.get(s);
                    if (n7 == null) {
                        if (FontConfiguration.logger != null) {
                            FontConfiguration.logger.config("Unrecognizable logicfont name " + s);
                        }
                        return;
                    }
                    else {
                        value[(int)n7] = access$1500;
                    }
                }
                this.sequences.put(n6, value);
                if (!(n4 == 0)) {
                    this.alphabeticSuffix.put(n6, Short.valueOf(FontConfiguration.access$1000("default")));
                }
                else if (!(n5 == 0)) {
                    this.alphabeticSuffix.put(n6, Short.valueOf(FontConfiguration.access$1000("1252")));
                }
            }
            else if (s.startsWith("allfonts.")) {
                s = s.substring(9);
                if (s.endsWith(".motif")) {
                    s = s.substring(0, s.length() - 6);
                    this.scriptAllfontsMotif.put(this.getID(this.scriptIDs, s), this.getID(this.componentFontNameIDs, s2));
                }
                else {
                    this.scriptAllfonts.put(this.getID(this.scriptIDs, s), this.getID(this.componentFontNameIDs, s2));
                }
            }
            else if (s.startsWith("awtfontpath.")) {
                s = s.substring(12);
                this.awtfontpaths.put(this.getID(this.scriptIDs, s), Short.valueOf(FontConfiguration.access$1000(s2)));
            }
            else if ("version".equals(s)) {
                this.version = s2;
            }
            else if ("appendedfontpath".equals(s)) {
                this.appendedfontpath = s2;
            }
            else if (s.startsWith("proportional.")) {
                s = s.substring(13).replace('_', ' ');
                this.proportionals.put(this.getID(this.componentFontNameIDs, s), this.getID(this.componentFontNameIDs, s2));
            }
            else {
                boolean n = false;
                int index = s.indexOf(46);
                if (index == -1) {
                    if (FontConfiguration.logger != null) {
                        FontConfiguration.logger.config("Failed parsing " + s + " property of font configuration.");
                    }
                    return;
                }
                else {
                    int index2 = s.indexOf(46, index + 1);
                    if (index2 == -1) {
                        if (FontConfiguration.logger != null) {
                            FontConfiguration.logger.config("Failed parsing " + s + " property of font configuration.");
                        }
                        return;
                    }
                    else {
                        if (s.endsWith(".motif")) {
                            s = s.substring(0, s.length() - 6);
                            n = true;
                        }
                        Integer n2 = (Integer)this.logicalFontIDs.get(s.substring(0, index));
                        Integer n3 = (Integer)this.fontStyleIDs.get(s.substring(index + 1, index2));
                        Short id = this.getID(this.scriptIDs, s.substring(index2 + 1));
                        if (n2 == null || n3 == null) {
                            if (FontConfiguration.logger != null) {
                                FontConfiguration.logger.config("unrecognizable logicfont name/style at " + s);
                            }
                            return;
                        }
                        else {
                            Short[] array;
                            if (!(n == 0)) {
                                array = (Short[])this.scriptFontsMotif.get(id);
                            }
                            else {
                                array = (Short[])this.scriptFonts.get(id);
                            }
                            if (array == null) {
                                array = new Short[20];
                            }
                            array[(int)n2 * 4 + (int)n3] = this.getID(this.componentFontNameIDs, s2);
                            if (!(n == 0)) {
                                this.scriptFontsMotif.put(id, array);
                            }
                            else {
                                this.scriptFonts.put(id, array);
                            }
                        }
                    }
                }
            }
        }
        
        static void access$2400(FontConfiguration.PropertiesHandler propertiesHandler, String s, String s2) {
            propertiesHandler.parseProperty(s, s2);
        }
        
        class FontProperties extends Properties
        {
            @Override
            public synchronized Object put(Object o, Object o2) {
                FontConfiguration.PropertiesHandler.access$2400(FontConfiguration.PropertiesHandler.this, (String)o, (String)o2);
                return null;
            }
        }
    }
}
